/*
    Project: TCGINA
    Version: V1.30
    File   : reg.c

    Copyright (c) 2005-2012 Author of TCGINA DLL. All rights reserved.
    Public Key Fingerprint: 294B A769 4A0A CC05 DAE6 00DD FF47 8C72 4097 67CE

    Covered by TrueCrypt License 2.3 the full text of which is contained
    in the file License.txt included in TCGINA binary and source code
    distribution archives.

    The copyright notice of this source code file and the public key
    fingerprint must be retained in full.
*/

#include <windows.h>
#include <tchar.h>
#include "misc.h"

BOOL RegKeyExists(HKEY hRootKey, LPCTSTR pszKeyName)
{
    HKEY hKey;

    if (RegOpenKeyEx(hRootKey, pszKeyName, 0, KEY_READ, &hKey) ==
      ERROR_SUCCESS) {
        RegCloseKey(hKey);
        return TRUE;
    }

    return FALSE;
}

BOOL RegKeyIsEmpty(HKEY hRootKey, LPCTSTR pszKeyName)
{
    HKEY  hKey;
    BOOL  fResult;
    DWORD cSubKeys;
    DWORD cValues;

    if (RegOpenKeyEx(hRootKey, pszKeyName, 0, KEY_READ, &hKey) !=
      ERROR_SUCCESS) {
        return FALSE;
    }

    fResult = FALSE;
    if (RegQueryInfoKey(hKey, NULL, NULL, NULL, &cSubKeys,
      NULL, NULL, &cValues, NULL, NULL, NULL, NULL) == ERROR_SUCCESS) {
        if (cSubKeys == 0 && cValues == 0) {
            fResult = TRUE;
        }
    }
    RegCloseKey(hKey);

    return fResult;
}

BOOL RegValueExists(HKEY hRootKey, LPCTSTR pszKeyName, LPCTSTR pszValueName)
{
    HKEY hKey;
    BOOL fResult = FALSE;

    if (RegOpenKeyEx(hRootKey, pszKeyName, 0, KEY_READ, &hKey) ==
      ERROR_SUCCESS) {
        DWORD cbSize;
        DWORD dwType;
        LONG  lResult;

        cbSize = 0;
        lResult = RegQueryValueEx(hKey, pszValueName, NULL, &dwType,
          NULL, &cbSize);
        if (lResult == ERROR_SUCCESS || lResult == ERROR_MORE_DATA) {
            fResult = TRUE;
        }

        RegCloseKey(hKey);
    }

    return fResult;
}

BOOL RegRemoveKey(HKEY hRootKey, LPCTSTR pszKeyName)
{
    HKEY     hKey;
    LONG     lResult;
    TCHAR    szSubKeyName[MAX_PATH];
    DWORD    dwSize;
    FILETIME ftLastWriteTime;

    if (RegDeleteKey(hRootKey, pszKeyName) == ERROR_SUCCESS) {
        return TRUE;
    }

    lResult = RegOpenKeyEx(hRootKey, pszKeyName, 0, KEY_READ, &hKey);
    if (lResult != ERROR_SUCCESS) {
        return (lResult == ERROR_FILE_NOT_FOUND) ? TRUE : FALSE;
    }

    dwSize = countof(szSubKeyName);
    while (RegEnumKeyEx(hKey, 0, szSubKeyName, &dwSize, NULL,
      NULL, NULL, &ftLastWriteTime) == ERROR_SUCCESS) {
        if (!RegRemoveKey(hKey, szSubKeyName)) {
            RegCloseKey(hKey);
            return FALSE;
        }
        dwSize = countof(szSubKeyName);
    }
    RegCloseKey(hKey);

    return RegDeleteKey(hRootKey, pszKeyName) == ERROR_SUCCESS;
}

BOOL RegRemoveValue(HKEY hRootKey, LPCTSTR pszKeyName,
  LPCTSTR pszValueName)
{
    HKEY hKey;
    BOOL fResult = FALSE;

    if (RegOpenKeyEx(hRootKey, pszKeyName, 0, KEY_WRITE, &hKey) ==
      ERROR_SUCCESS) {
        if (RegDeleteValue(hKey, pszValueName) == ERROR_SUCCESS) {
            fResult = TRUE;
        }
        RegCloseKey(hKey);
    }

    return fResult;
}

BOOL RegCopyValue(HKEY hSrcRootKey, LPCTSTR pszSrcKeyName,
  LPCTSTR pszSrcValueName, HKEY hDestRootKey, LPCTSTR pszDestKeyName,
  LPCTSTR pszDestValueName)
{
    HKEY hKey;
    void *pData;
    DWORD cbSize;
    DWORD dwType;
    LONG  lResult;

    if (RegOpenKeyEx(hSrcRootKey, pszSrcKeyName, 0, KEY_READ, &hKey)
      != ERROR_SUCCESS) {
        return FALSE;
    }

    cbSize = 0;
    lResult = RegQueryValueEx(hKey, pszSrcValueName, NULL, &dwType,
      NULL, &cbSize);
    if (lResult != ERROR_SUCCESS && lResult != ERROR_MORE_DATA) {
        RegCloseKey(hKey);
        return FALSE;
    }
    pData = malloc(cbSize+1);
    if (pData == NULL) {
        RegCloseKey(hKey);
        return FALSE;
    }
    if (RegQueryValueEx(hKey, pszSrcValueName, NULL, &dwType,
      pData, &cbSize) != ERROR_SUCCESS) {
        free(pData);
        RegCloseKey(hKey);
        return FALSE;
    }
    RegCloseKey(hKey);

    if (RegCreateKeyEx(hDestRootKey, pszDestKeyName, 0, NULL, 0,
      KEY_WRITE, NULL, &hKey, NULL) != ERROR_SUCCESS) {
        free(pData);
        return FALSE;
    }
    if (RegSetValueEx(hKey, pszDestValueName, 0, dwType, pData,
      cbSize) != ERROR_SUCCESS) {
        free(pData);
        RegCloseKey(hKey);
        return FALSE;
    }

    free(pData);
    RegCloseKey(hKey);

    return TRUE;
}

BOOL RegCloneValue(HKEY hSrcRootKey, LPCTSTR pszSrcKeyName,
  LPCTSTR pszSrcValueName, HKEY hDestRootKey, LPCTSTR pszDestKeyName,
  LPCTSTR pszDestValueName) {

    if (!RegValueExists(hSrcRootKey, pszSrcKeyName, pszSrcValueName)) {
        if (!RegValueExists(hDestRootKey, pszDestKeyName, pszDestValueName)) {
            return TRUE;
        }
        return RegRemoveValue(hDestRootKey, pszDestKeyName, pszDestValueName);
    }

    return RegCopyValue(hSrcRootKey, pszSrcKeyName, pszSrcValueName,
      hDestRootKey, pszDestKeyName, pszDestValueName);
}

BOOL RegGetValueData(HKEY hRootKey, LPCTSTR pszKeyName,
  LPCTSTR pszValueName, PDWORD pdwType, PBYTE pDest, PDWORD pcbSize)
{
    HKEY  hKey;
    BOOL  fResult;

    fResult = FALSE;
    if (RegOpenKeyEx(hRootKey, pszKeyName, 0, KEY_READ, &hKey) ==
      ERROR_SUCCESS) {
        if (RegQueryValueEx(hKey, pszValueName, NULL, pdwType,
          pDest, pcbSize) == ERROR_SUCCESS) {
            fResult = TRUE;
        }
        RegCloseKey(hKey);
    }

    return fResult;
}

BOOL RegSetString(HKEY hRootKey, LPCTSTR pszKeyName, LPCTSTR pszValueName,
  DWORD dwType, LPCTSTR pszValueData)
{
    HKEY hKey;
    BOOL  fResult = FALSE;

    if (RegCreateKeyEx(hRootKey, pszKeyName, 0, NULL, 0,
      KEY_WRITE, NULL, &hKey, NULL) != ERROR_SUCCESS) {
        return FALSE;
    }
    if (RegSetValueEx(hKey, pszValueName, 0, dwType, (const BYTE*)pszValueData,
      (DWORD)(_tcslen(pszValueData)+1)*sizeof(TCHAR)) == ERROR_SUCCESS) {
        fResult = TRUE;
    }
    RegCloseKey(hKey);

    return fResult;
}

BOOL RegSetInteger(HKEY hRootKey, LPCTSTR pszKeyName, LPCTSTR pszValueName,
  DWORD dwData)
{
    HKEY hKey;
    BOOL fResult = FALSE;

    if (RegCreateKeyEx(hRootKey, pszKeyName, 0, NULL, 0,
      KEY_WRITE, NULL, &hKey, NULL) != ERROR_SUCCESS) {
        return FALSE;
    }
    if (RegSetValueEx(hKey, pszValueName, 0, REG_DWORD, (const BYTE*)&dwData,
      sizeof(DWORD)) == ERROR_SUCCESS) {
        fResult = TRUE;
    }
    RegCloseKey(hKey);

    return fResult;
}

BOOL PendingFileRenameOperationsContains(LPCTSTR pszUpperCaseSubString)
{
    HKEY   hKey;
    DWORD  dwType;
    DWORD  cbSize;
    LPTSTR pszBuffer;
    LPTSTR pszSrcFile;
    LPTSTR pszDestFile;

    if (RegOpenKeyEx(HKEY_LOCAL_MACHINE,
      TEXT("SYSTEM\\CurrentControlSet\\Control\\Session Manager"),
      0, KEY_READ, &hKey) != ERROR_SUCCESS) {
        return FALSE;
    }
    dwType = REG_NONE;
    cbSize = 0;
    RegQueryValueEx(hKey, TEXT("PendingFileRenameOperations"), NULL, &dwType,
      NULL, &cbSize);
    if (cbSize == 0 || dwType != REG_MULTI_SZ) {
        RegCloseKey(hKey);
        return FALSE;
    }

    if (cbSize > 1024*1024) {
        RegCloseKey(hKey);
        return TRUE;
    }
    pszBuffer = LocalAlloc(LMEM_FIXED|LMEM_ZEROINIT, cbSize + 2*sizeof(TCHAR));
    if (pszBuffer == NULL) {
        RegCloseKey(hKey);
        return TRUE;
    }
    if (RegQueryValueEx(hKey, TEXT("PendingFileRenameOperations"), NULL, &dwType,
      (LPBYTE)pszBuffer, &cbSize) != ERROR_SUCCESS) {
        LocalFree(pszBuffer);
        RegCloseKey(hKey);
        return TRUE;
    }
    RegCloseKey(hKey);

    pszSrcFile = pszBuffer;
    for (;;) {
        pszDestFile = pszSrcFile + (_tcslen(pszSrcFile) + 1);
        if (*pszSrcFile == TEXT('\0') && *pszDestFile == TEXT('\0')) {
            break;
        }
        _tcsupr(pszSrcFile);
        _tcsupr(pszDestFile);
        if (_tcsstr(pszSrcFile, pszUpperCaseSubString) != NULL ||
          _tcsstr(pszDestFile, pszUpperCaseSubString) != NULL) {
            LocalFree(pszBuffer);
            return TRUE;
        }
        pszSrcFile = pszDestFile + (_tcslen(pszDestFile) + 1);
    }

    LocalFree(pszBuffer);

    return FALSE;
}
